/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "lpc17xx_pinsel.h"
#include "lpc17xx_gpio.h"
#include "lpc17xx_ssp.h"
#include "lpc17xx_uart.h"
#include "lpc17xx_timer.h"
#include "lpc17xx_can.h"
#include <string.h>
#include <stdio.h>

#include "board.h"
#include "time.h"
#include "httpd.h"
#include "eadebug.h"

#include "diskio.h"
#include "ff.h"

/******************************************************************************
 * Local variables
 *****************************************************************************/

/* IP address of the webserver */
static uint8_t ip[4] = {192,168,5,239};
/* gateway IP address */
static uint8_t gw[4] = {192,168,5,1};
/* network mask */
static uint8_t mask[4] = {255,255,255,0};

static FATFS Fatfs[1];
static uint32_t fatfsTime = 0;

/******************************************************************************
 * Public Functions
 *****************************************************************************/

DWORD get_fattime (void)
{
    return 0;
}


int main (void)
{
  DSTATUS stat;
  FRESULT result;
  uint8_t res = 0;

  time_init();
  ssp1_init();

  console_init();
  console_sendString((uint8_t*)"LWIP - webserver demo\r\n");


  res = net_init(ip, mask, gw);
  if (res < 0) {
    console_sendString((uint8_t*)"Failed to init network\r\n");
    return 0;
  }

  console_sendString((uint8_t*)"Initializing file system\r\n");


  stat = disk_initialize(0);
  if (stat & STA_NOINIT) {
    console_sendString((uint8_t*)"SD: not initialized\r\n");
  }

  if (stat & STA_NODISK) {
    console_sendString((uint8_t*)"SD: No Disk\r\n");
  }

  if (stat != 0) {
    return 0;
  }

  result = f_mount(0, &Fatfs[0]);
  if (res != FR_OK) {
    console_sendString((uint8_t*)"SD: failed to mount\r\n");
    return 0;
  }

  httpd_init();

  console_sendString((uint8_t*)"Web server started\r\n");

  while (1) {
    net_task();
    if (fatfsTime < time_get()) {
      // disk_timerproc should be called at a 10 ms period
      fatfsTime = time_get() + 10;
      disk_timerproc();
    }
  }


}

void check_failed(uint8_t *file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while(1);
}

